@@ -1,6 +1,6 @@
 /*
   Simple DirectMedia Layer
-  Copyright (C) 1997-2013 Sam Lantinga <slouken@libsdl.org>
+  Copyright (C) 1997-2014 Sam Lantinga <slouken@libsdl.org>
 
   This software is provided 'as-is', without any express or implied
   warranty.  In no event will the authors be held liable for any damages
@@ -25,7 +25,7 @@
    was the cleanest way to move it to 1.3. The 1.2 target was written by
    Stéphan Kochen: stephan .a.t. kochen.nl
 */
-#include "SDL_config.h"
+#include "../../SDL_internal.h"
 
 #if SDL_AUDIO_DRIVER_PULSEAUDIO
 
@@ -110,7 +110,9 @@ static int load_pulseaudio_syms(void);
 #ifdef SDL_AUDIO_DRIVER_PULSEAUDIO_DYNAMIC
 
 static const char *pulseaudio_library = SDL_AUDIO_DRIVER_PULSEAUDIO_DYNAMIC;
+static const char *pulsesimple_library = SDL_AUDIO_DRIVER_PULSESIMPLE_DYNAMIC;
 static void *pulseaudio_handle = NULL;
+static void *pulsesimple_handle = NULL;
 
 static int
 load_pulseaudio_sym(const char *fn, void **addr)
@@ -124,14 +126,29 @@ load_pulseaudio_sym(const char *fn, void **addr)
     return 1;
 }
 
+static int
+load_pulsesimple_sym(const char *fn, void **addr)
+{
+    *addr = SDL_LoadFunction(pulsesimple_handle, fn);
+    if (*addr == NULL) {
+        /* Don't call SDL_SetError(): SDL_LoadFunction already did. */
+        return 0;
+    }
+
+    return 1;
+}
+
 /* cast funcs to char* first, to please GCC's strict aliasing rules. */
 #define SDL_PULSEAUDIO_SYM(x) \
     if (!load_pulseaudio_sym(#x, (void **) (char *) &PULSEAUDIO_##x)) return -1
+#define SDL_PULSESIMPLE_SYM(x) \
+    if (!load_pulsesimple_sym(#x, (void **) (char *) &PULSEAUDIO_##x)) return -1
 
 static void
 UnloadPulseAudioLibrary(void)
 {
     if (pulseaudio_handle != NULL) {
+        SDL_UnloadObject(pulsesimple_handle);
         SDL_UnloadObject(pulseaudio_handle);
         pulseaudio_handle = NULL;
     }
@@ -143,7 +160,8 @@ LoadPulseAudioLibrary(void)
     int retval = 0;
     if (pulseaudio_handle == NULL) {
         pulseaudio_handle = SDL_LoadObject(pulseaudio_library);
-        if (pulseaudio_handle == NULL) {
+        pulsesimple_handle = SDL_LoadObject(pulsesimple_library);
+        if (pulseaudio_handle == NULL || pulsesimple_handle == NULL) {
             retval = -1;
             /* Don't call SDL_SetError(): SDL_LoadObject already did. */
         } else {
@@ -159,6 +177,7 @@ LoadPulseAudioLibrary(void)
 #else
 
 #define SDL_PULSEAUDIO_SYM(x) PULSEAUDIO_##x = x
+#define SDL_PULSESIMPLE_SYM(x) PULSEAUDIO_##x = x
 
 static void
 UnloadPulseAudioLibrary(void)
@@ -179,8 +198,8 @@ static int
 load_pulseaudio_syms(void)
 {
     SDL_PULSEAUDIO_SYM(pa_get_library_version);
-    SDL_PULSEAUDIO_SYM(pa_simple_new);
-    SDL_PULSEAUDIO_SYM(pa_simple_free);
+    SDL_PULSESIMPLE_SYM(pa_simple_new);
+    SDL_PULSESIMPLE_SYM(pa_simple_free);
     SDL_PULSEAUDIO_SYM(pa_mainloop_new);
     SDL_PULSEAUDIO_SYM(pa_mainloop_get_api);
     SDL_PULSEAUDIO_SYM(pa_mainloop_iterate);
